/*
 * query.h
 *
 * Copyright (C) 2009 Tim Marston <edam@waxworlds.org>
 *
 * This file is part of sqlite3cc (hereafter referred to as "this program").
 * See http://www.waxworlds.org/edam/software/sqlite3cc for more information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SQLITE3CC_QUERY_H_
#define SQLITE3CC_QUERY_H_


#include <boost/iterator/iterator_facade.hpp>
#include <sqlite3cc/basic_statement.h>
#include <sqlite3cc/row.h>


namespace sqlite
{


class query
	:
	public basic_statement
{
//______________________________________________________________________________
//                                                                 instantiation
public:

	/**
	 * Constructor that provides a database upon which to act and the SQL
	 * query to execute.
	 * @param database a reference to a database
	 * @param sql an SQL statement in UTF-8
	 */
	explicit query(
		database &database,
		const std::string &sql );

	/**
	 * Constructor that provides a database upon which to act.
	 * @param database a reference to a database
	 * @param sql an SQL statement in UTF-8
	 */
	explicit query(
		database &database );

//______________________________________________________________________________
//                                                              public interface
public:

	/**
	 * Prepare an SQL statement.
	 * @param sql an SQL statement in UTF-8
	 * @returns an sqlite error code
	 * @see sqlite3_prepare_v2()
	 */
	virtual int prepare(
		const std::string &sql );

	/**
	 * Perform a step() and return row object that can be used to retrieve the
	 * results.
	 * @return a row object
	 */
	row step();

	/**
	 * Get the number of columns in the results
	 * @see sqlite3_column_count()
	 */
	unsigned int column_count();

	/**
	 * Get the name of a column in the results
	 * @param index column index
	 * @see sqlite3_column_name()
	 */
	const std::string column_name(
		unsigned int index );

	/**
	 * Query iterator which can be used to obtain rows
	 */
	class iterator
		:
		public boost::iterator_facade< iterator, row,
			boost::single_pass_traversal_tag, row >
	{
	public:
		iterator();
		explicit iterator( query &query, bool valid );

	private:
		friend class boost::iterator_core_access;

		row dereference() const;
		void increment();
		bool equal( iterator const &other ) const;

		/** is this iterator still pointing to a valid row? */
		bool _valid;

		/** the query */
		query &_query;
	};

	/**
	 * Return an iterator to the initial row. Note that creating an iterator
	 * causes step() to be called, so it should only be called to begin
	 * iterating over the rows and not for comparison.
	 * @return a query iterator
	 */
	iterator begin();

	/**
	 * Return an iterator to after the last row (i.e., an invalid row).
	 */
	iterator end();

//______________________________________________________________________________
//                                                                implementation
private:

	/** next row index */
	unsigned long long _next_row;

};


} // namespace sqlite


#endif /* SQLITE3CC_QUERY_H_ */
