/*
 * query.cc
 *
 * Copyright (C) 2009 Tim Marston <edam@waxworlds.org>
 *
 * This file is part of sqlite3cc (hereafter referred to as "this program").
 * See http://www.waxworlds.org/edam/software/sqlite3cc for more information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sqlite3cc/query.h>
#include <sqlite3cc/row.h>
#include <sqlite3cc/database.h>
#include <assert.h>


sqlite::query::query(
	database &database,
	const std::string &sql )
	:
	basic_statement( database, sql ),
	_next_row( 0 )
{
	assert( sqlite3_column_count( _handle ) > 0 );
}


sqlite::query::query(
	database &database )
	:
	basic_statement( database ),
	_next_row( 0 )
{
}


int sqlite::query::prepare(
	const std::string &sql )
{
	_next_row = 0;
	int code = basic_statement::prepare( sql );
	assert( code != SQLITE_OK || sqlite3_column_count( _handle ) > 0 );
	return code;
}


sqlite::row sqlite::query::step()
{
	database::database_mutex_guard lock( _database );

	int code = basic_statement::step();
	if( code == SQLITE_DONE ) return row();
	if( code == SQLITE_ROW ) return row( _handle, _next_row++ );

	throw sqlite_error( _database, code );
}


unsigned int sqlite::query::column_count()
{
	return sqlite3_column_count( _handle );
}


const std::string sqlite::query::column_name(
	unsigned int index )
{
	assert( index <
		static_cast< unsigned int >( sqlite3_column_count( _handle ) ) );
	return sqlite3_column_name( _handle, index );
}


sqlite::query::iterator::iterator(
	sqlite::query &query,
	bool valid )
	:
	_query( query )
{
	if( valid )
		increment();
	else
		_valid = false;
}


sqlite::row sqlite::query::iterator::dereference() const
{
	return sqlite::row( _query._handle, _query._next_row++ );
}


void sqlite::query::iterator::increment()
{
	_valid = _query.step();
}


bool sqlite::query::iterator::equal(
	sqlite::query::iterator const &other )
	const
{
	return _valid == other._valid;
}


sqlite::query::iterator sqlite::query::begin()
{
	return sqlite::query::iterator( *this, true );
}


sqlite::query::iterator sqlite::query::end()
{
	return sqlite::query::iterator( *this, false );
}

